<?php

namespace App\Repositories;

use App\Models\Track;
use Sunra\PhpSimple\HtmlDomParser;
use Yangqi\Htmldom\Htmldom;

class TrackRepository extends BaseRepository
{
    public function __construct()
    {

    }

    /**
     * Lấy ds bài hát đã insert
     *
     * doanpv - 24/03/2020
     *
     * @param string $pluck_by_field
     * @param string $type
     * @param array $select_fields
     *
     * @return array
     */
    public function getTrackInserted($pluck_by_field='all', $type='original', array $select_fields)
    {
        if (count($select_fields) == 0) $select_fields = ['id', 'title', 'slug'];

        $query = Track::select($select_fields);

        if ($pluck_by_field == 'all') {
            return $query->get()->toArray();
        }

        switch ($type) {
            case 'upper':
                return $query->pluck($pluck_by_field)->map(function ($item){
                    return strtoupper($item);
                })->all();
                break;
            case 'lower':
                return $query->pluck($pluck_by_field)->map(function ($item){
                    return strtolower($item);
                })->all();
                break;
            default: return $query->pluck($pluck_by_field)->toArray();

        }
    }

    /**
     * Lấy ds bài hát chưa có ảnh đại diện (trường art: null)
     *
     * doanpv - 24/03/2020
     *
     * @return array
     */
    public function getTrackImageEmpty()
    {
        $query = Track::select(['id', 'art', 'id_zing', 'id_nct', 'id_keeng', 'src_thumbnail_medium'])
            ->whereNull('art')
            ->whereNotNull('src_thumbnail_medium');

        return $query->get()->toArray();
    }

    public function processNctGetTrackInfoFromTop100($url)
    {
        $tracks = [];
        $dom = getDom($url);

        /* Lấy tên bài hát */
        $element_track_title = 'div.box_info_field h3.h3 a';
        foreach ($dom->find($element_track_title) as $data) {
            $tracks['title'][] = str_replace(['<h2 class="card-header">', '</h2>'], ['', ''], $data->innertext);
        }

        /* Lấy đường dẫn ảnh của bài hát */
        $element_track_img = 'div.box_info_field a img';
        foreach ($dom->find($element_track_img) as $data) {
            $tracks['img'][] = str_replace(['<h2 class="card-header">', '</h2>'], ['', ''], $data->{"data-src"});
        }

        /* Lấy link bài hát */
        $element_track_link = 'div.box_info_field h3.h3 a';
        foreach ($dom->find($element_track_link) as $data) {
            $link = str_replace(['<h2 class="card-header">', '</h2>'], ['', ''], $data->href);
            $tracks['link'][] = $link;
        }

        /* Lấy tên ca sĩ hát */
        $element_track_singer = 'div.box_info_field h4.list_name_singer a';
        foreach ($dom->find($element_track_singer) as $data) {
            $tracks['singer'][] = str_replace(['<h2 class="card-header">', '</h2>'], ['', ''], $data->innertext);
        }

        /* Lấy url thông tin cá nhân ca sĩ */
        $element_singer_profile = 'div.box_info_field h4.list_name_singer a';
        foreach ($dom->find($element_singer_profile) as $data) {
            $tracks['url_profile'][] = str_replace(['<h2 class="card-header">', '</h2>'], ['', ''], $data->href);
        }

        /* Lấy ID dạng chuỗi của bài hát */
        $element_track_id_str = 'div.box_song_action a.button_add_playlist';
        foreach ($dom->find($element_track_id_str) as $data) {
            $tracks['id_nct_str'][] = str_replace(['btnShowBoxPlaylist_', '</h2>'], ['', ''], $data->id);
        }

        /* Lấy ID dạng số của bài hát */
        $element_track_id = 'div.box_info_field';
        $start = '<span class=\"export_listen\" id="NCTCounter__';
        $end = '">0<\/span>';
        $rule = "/$start(.*)$end/msU";
        foreach ($dom->find($element_track_id) as $data) {
            $data_crawler = str_replace(['<!--', '-->'], ['', ''], $data);
            preg_match($rule, $data_crawler, $result);
            if (is_array($result) && count($result) == 2) {
                $tracks['id_nct'][] = $result[1];
            }
        }

        return $tracks;
    }

    public function processNctGetTrackInfoFromTop100Playlist($url)
    {
        $tracks = [];
        $dom = new Htmldom($url);

        /* Lấy ID dạng chuỗi của bài hát */
        $element_track_id_str = 'li';
        foreach ($dom->find($element_track_id_str) as $data) {
            if (isset($data->key)) {
                $tracks['id_nct_str'][] = str_replace([], [], $data->key);
            }
        }

        /* Lấy link bài hát */
        $element_track_link = 'li a';
        foreach ($dom->find($element_track_link) as $data) {
            if (isset($data->class) && $data->class == 'button_new_window') {
                $tracks['link'][] = str_replace([], [], $data->href);
            }
        }

        /* Lấy ID dạng số */
        $element_track_id = 'li span';
        foreach ($dom->find($element_track_id) as $data) {
            if (isset($data->id) && isset($data->class) && $data->class == 'icon_listen') {
                $tracks['id_nct'][] = str_replace(['NCTCounter_sg_', 'NCTCounter_pn_'], ['', ''], $data->id);
            }
        }

        return $tracks;
    }


}